@@ -4,7 +4,7 @@
 # checkbutton, and radiobutton widgets and provides procedures
 # that help in implementing those bindings.
 #
-# RCS: @(#) $Id: button.tcl,v 1.17 2002/09/04 02:05:52 hobbs Exp $
+# RCS: @(#) $Id: button.tcl,v 1.19.4.1 2009/10/24 00:12:03 dkf Exp $
 #
 # Copyright (c) 1992-1994 The Regents of the University of California.
 # Copyright (c) 1994-1996 Sun Microsystems, Inc.
@@ -18,8 +18,7 @@
 # The code below creates the default class bindings for buttons.
 #-------------------------------------------------------------------------
 
-if {[string equal [tk windowingsystem] "classic"]
-	|| [string equal [tk windowingsystem] "aqua"]} {
+if {[tk windowingsystem] eq "aqua"} {
     bind Radiobutton <Enter> {
 	tk::ButtonEnter %W
     }
@@ -38,8 +37,11 @@ if {[string equal [tk windowingsystem] "classic"]
     bind Checkbutton <ButtonRelease-1> {
 	tk::ButtonUp %W
     }
+    bind Checkbutton <Leave> {
+	tk::ButtonLeave %W
+    }
 }
-if {[string equal "windows" $tcl_platform(platform)]} {
+if {"windows" eq $tcl_platform(platform)} {
     bind Checkbutton <equal> {
 	tk::CheckRadioInvoke %W select
     }
@@ -58,6 +60,9 @@ if {[string equal "windows" $tcl_platform(platform)]} {
     bind Checkbutton <Enter> {
 	tk::CheckRadioEnter %W
     }
+    bind Checkbutton <Leave> {
+	tk::ButtonLeave %W
+    }
 
     bind Radiobutton <1> {
 	tk::CheckRadioDown %W
@@ -69,10 +74,10 @@ if {[string equal "windows" $tcl_platform(platform)]} {
 	tk::CheckRadioEnter %W
     }
 }
-if {[string equal "x11" [tk windowingsystem]]} {
+if {"x11" eq [tk windowingsystem]} {
     bind Checkbutton <Return> {
 	if {!$tk_strictMotif} {
-	    tk::CheckRadioInvoke %W
+	    tk::CheckInvoke %W
 	}
     }
     bind Radiobutton <Return> {
@@ -81,17 +86,20 @@ if {[string equal "x11" [tk windowingsystem]]} {
 	}
     }
     bind Checkbutton <1> {
-	tk::CheckRadioInvoke %W
+	tk::CheckInvoke %W
     }
     bind Radiobutton <1> {
 	tk::CheckRadioInvoke %W
     }
     bind Checkbutton <Enter> {
-	tk::ButtonEnter %W
+	tk::CheckEnter %W
     }
     bind Radiobutton <Enter> {
 	tk::ButtonEnter %W
     }
+    bind Checkbutton <Leave> {
+	tk::CheckLeave %W
+    }
 }
 
 bind Button <space> {
@@ -119,16 +127,13 @@ bind Button <ButtonRelease-1> {
 }
 
 bind Checkbutton <FocusIn> {}
-bind Checkbutton <Leave> {
-    tk::ButtonLeave %W
-}
 
 bind Radiobutton <FocusIn> {}
 bind Radiobutton <Leave> {
     tk::ButtonLeave %W
 }
 
-if {[string equal "windows" $tcl_platform(platform)]} {
+if {"windows" eq $tcl_platform(platform)} {
 
 #########################
 # Windows implementation 
@@ -309,7 +314,7 @@ proc ::tk::CheckRadioDown w {
 
 }
 
-if {[string equal "x11" [tk windowingsystem]]} {
+if {"x11" eq [tk windowingsystem]} {
 
 #####################
 # Unix implementation
@@ -417,7 +422,7 @@ proc ::tk::ButtonDown w {
 
 proc ::tk::ButtonUp w {
     variable ::tk::Priv
-    if {[string equal $w $Priv(buttonWindow)]} {
+    if {$w eq $Priv(buttonWindow)} {
 	set Priv(buttonWindow) ""
 
 	# Restore the button's relief if it was cached.
@@ -445,8 +450,7 @@ proc ::tk::ButtonUp w {
 
 }
 
-if {[string equal [tk windowingsystem] "classic"]
-	|| [string equal [tk windowingsystem] "aqua"]} {
+if {[tk windowingsystem] eq "aqua"} {
 
 ####################
 # Mac implementation
@@ -637,3 +641,108 @@ proc ::tk::CheckRadioInvoke {w {cmd invoke}} {
 	uplevel #0 [list $w $cmd]
     }
 }
+
+# Special versions of the handlers for checkbuttons on Unix that do the magic
+# to make things work right when the checkbutton indicator is hidden;
+# radiobuttons don't need this complexity.
+
+# ::tk::CheckInvoke --
+# The procedure below invokes the checkbutton, like ButtonInvoke, but handles
+# what to do when the checkbutton indicator is missing. Only used on Unix.
+#
+# Arguments:
+# w -		The name of the widget.
+
+proc ::tk::CheckInvoke {w} {
+    variable ::tk::Priv
+    if {[$w cget -state] ne "disabled"} {
+	# Additional logic to switch the "selected" colors around if necessary
+	# (when we're indicator-less).
+
+	if {![$w cget -indicatoron]} {
+	    if {[$w cget -selectcolor] eq $Priv($w,aselectcolor)} {
+		$w configure -selectcolor $Priv($w,selectcolor)
+	    } else {
+		$w configure -selectcolor $Priv($w,aselectcolor)
+	    }
+	}
+	uplevel #0 [list $w invoke]
+    }
+}
+
+# ::tk::CheckEnter --
+# The procedure below enters the checkbutton, like ButtonEnter, but handles
+# what to do when the checkbutton indicator is missing. Only used on Unix.
+#
+# Arguments:
+# w -		The name of the widget.
+
+proc ::tk::CheckEnter {w} {
+    variable ::tk::Priv
+    if {[$w cget -state] ne "disabled"} {
+	# On unix the state is active just with mouse-over
+	$w configure -state active
+
+	# If the mouse button is down, set the relief to sunken on entry.
+	# Overwise, if there's an -overrelief value, set the relief to that.
+
+	set Priv($w,relief) [$w cget -relief]
+	if {$Priv(buttonWindow) eq $w} {
+	    $w configure -relief sunken
+	    set Priv($w,prelief) sunken
+	} elseif {[set over [$w cget -overrelief]] ne ""} {
+	    $w configure -relief $over
+	    set Priv($w,prelief) $over
+	}
+
+	# Compute what the "selected and active" color should be.
+
+	if {![$w cget -indicatoron]} {
+	    set Priv($w,selectcolor) [$w cget -selectcolor]
+	    lassign [winfo rgb $w [$w cget -selectcolor]]      r1 g1 b1
+	    lassign [winfo rgb $w [$w cget -activebackground]] r2 g2 b2
+	    set Priv($w,aselectcolor) \
+		[format "#%04x%04x%04x" [expr {($r1+$r2)/2}] \
+		     [expr {($g1+$g2)/2}] [expr {($b1+$b2)/2}]]
+	    if {[set ::[$w cget -variable]] eq [$w cget -onvalue]} {
+		$w configure -selectcolor $Priv($w,aselectcolor)
+	    }
+	}
+    }
+    set Priv(window) $w
+}
+
+# ::tk::CheckLeave --
+# The procedure below leaves the checkbutton, like ButtonLeave, but handles
+# what to do when the checkbutton indicator is missing. Only used on Unix.
+#
+# Arguments:
+# w -		The name of the widget.
+
+proc ::tk::CheckLeave {w} {
+    variable ::tk::Priv
+    if {[$w cget -state] ne "disabled"} {
+	$w configure -state normal
+    }
+
+    # Restore the original button "selected" color; assume that the user
+    # wasn't monkeying around with things too much.
+
+    if {![$w cget -indicatoron] && [info exist Priv($w,selectcolor)]} {
+	$w configure -selectcolor $Priv($w,selectcolor)
+    }
+    unset -nocomplain Priv($w,selectcolor) Priv($w,aselectcolor)
+
+    # Restore the original button relief if it was changed by Tk. That is
+    # signaled by the existence of Priv($w,prelief).
+
+    if {[info exists Priv($w,relief)]} {
+	if {[info exists Priv($w,prelief)] && \
+		$Priv($w,prelief) eq [$w cget -relief]} {
+	    $w configure -relief $Priv($w,relief)
+	}
+	unset -nocomplain Priv($w,relief) Priv($w,prelief)
+    }
+
+    set Priv(window) ""
+}
